@@ -14,6 +14,13 @@ module Adapters
     include Integration::Common
     include Integration::NonParallel
 
+    # Rack::MockResponse doesn't provide any way to access the reason phrase,
+    # so override the shared test from Common.
+    def test_GET_reason_phrase
+      response = get('echo')
+      assert_nil response.reason_phrase
+    end
+
     # not using shared test because error is swallowed by Sinatra
     def test_timeout
       conn = create_connection(:request => {:timeout => 1, :open_timeout => 1})
@@ -22,5 +29,10 @@ module Adapters
       rescue Faraday::Error::ClientError
       end
     end
+
+    # test not applicable
+    undef test_connection_error
+    undef test_proxy
+    undef test_proxy_auth_fail
   end
 end
